@@ -28,10 +28,11 @@
 #include "nghttp2_config.h"
 
 #ifndef _WIN32
-#include <sys/uio.h>
+#  include <sys/uio.h>
 #endif // !_WIN32
 
 #include <cassert>
+#include <utility>
 
 #include "template.h"
 
@@ -65,25 +66,19 @@ struct BlockAllocator {
   ~BlockAllocator() { reset(); }
 
   BlockAllocator(BlockAllocator &&other) noexcept
-      : retain(other.retain),
-        head(other.head),
+      : retain{std::exchange(other.retain, nullptr)},
+        head{std::exchange(other.head, nullptr)},
         block_size(other.block_size),
-        isolation_threshold(other.isolation_threshold) {
-    other.retain = nullptr;
-    other.head = nullptr;
-  }
+        isolation_threshold(other.isolation_threshold) {}
 
   BlockAllocator &operator=(BlockAllocator &&other) noexcept {
     reset();
 
-    retain = other.retain;
-    head = other.head;
+    retain = std::exchange(other.retain, nullptr);
+    head = std::exchange(other.head, nullptr);
     block_size = other.block_size;
     isolation_threshold = other.isolation_threshold;
 
-    other.retain = nullptr;
-    other.head = nullptr;
-
     return *this;
   }
 
@@ -195,14 +190,14 @@ StringRef make_string_ref(BlockAllocator &alloc, const StringRef &src) {
 
 // private function used in concat_string_ref.  this is the base
 // function of concat_string_ref_count().
-inline size_t concat_string_ref_count(size_t acc) { return acc; }
+inline constexpr size_t concat_string_ref_count(size_t acc) { return acc; }
 
 // private function used in concat_string_ref.  This function counts
 // the sum of length of given arguments.  The calculated length is
 // accumulated, and passed to the next function.
 template <typename... Args>
-size_t concat_string_ref_count(size_t acc, const StringRef &value,
-                               Args &&... args) {
+constexpr size_t concat_string_ref_count(size_t acc, const StringRef &value,
+                                         Args &&...args) {
   return concat_string_ref_count(acc + value.size(),
                                  std::forward<Args>(args)...);
 }
@@ -217,7 +212,7 @@ inline uint8_t *concat_string_ref_copy(uint8_t *p) { return p; }
 // beyond the last byte written.
 template <typename... Args>
 uint8_t *concat_string_ref_copy(uint8_t *p, const StringRef &value,
-                                Args &&... args) {
+                                Args &&...args) {
   p = std::copy(std::begin(value), std::end(value), p);
   return concat_string_ref_copy(p, std::forward<Args>(args)...);
 }
@@ -225,7 +220,7 @@ uint8_t *concat_string_ref_copy(uint8_t *p, const StringRef &value,
 // Returns the string which is the concatenation of |args| in the
 // given order.  The resulting string will be NULL-terminated.
 template <typename BlockAllocator, typename... Args>
-StringRef concat_string_ref(BlockAllocator &alloc, Args &&... args) {
+StringRef concat_string_ref(BlockAllocator &alloc, Args &&...args) {
   size_t len = concat_string_ref_count(0, std::forward<Args>(args)...);
   auto dst = static_cast<uint8_t *>(alloc.alloc(len + 1));
   auto p = dst;
@@ -242,7 +237,7 @@ StringRef concat_string_ref(BlockAllocator &alloc, Args &&... args) {
 // then just call concat_string_ref().
 template <typename BlockAllocator, typename... Args>
 StringRef realloc_concat_string_ref(BlockAllocator &alloc,
-                                    const StringRef &value, Args &&... args) {
+                                    const StringRef &value, Args &&...args) {
   if (value.empty()) {
     return concat_string_ref(alloc, std::forward<Args>(args)...);
   }
