@@ -41,17 +41,10 @@
 #include "ssl_compat.h"
 
 using namespace nghttp2;
+using namespace std::chrono_literals;
 
 namespace shrpx {
 
-#if !LIBRESSL_3_5_API && !LIBRESSL_2_7_API && !OPENSSL_1_1_API
-
-void *BIO_get_data(BIO *bio) { return bio->ptr; }
-void BIO_set_data(BIO *bio, void *ptr) { bio->ptr = ptr; }
-void BIO_set_init(BIO *bio, int init) { bio->init = init; }
-
-#endif // !LIBRESSL_3_5_API && !LIBRESSL_2_7_API && !OPENSSL_1_1_API
-
 Connection::Connection(struct ev_loop *loop, int fd, SSL *ssl,
                        MemchunkPool *mcpool, ev_tstamp write_timeout,
                        ev_tstamp read_timeout,
@@ -72,9 +65,8 @@ Connection::Connection(struct ev_loop *loop, int fd, SSL *ssl,
       data(data),
       fd(fd),
       tls_dyn_rec_warmup_threshold(tls_dyn_rec_warmup_threshold),
-      tls_dyn_rec_idle_timeout(tls_dyn_rec_idle_timeout),
+      tls_dyn_rec_idle_timeout(util::duration_from(tls_dyn_rec_idle_timeout)),
       proto(proto),
-      last_read(0.),
       read_timeout(read_timeout) {
 
   ev_io_init(&wev, writecb, fd, EV_WRITE);
@@ -89,9 +81,6 @@ Connection::Connection(struct ev_loop *loop, int fd, SSL *ssl,
   wt.data = this;
   rt.data = this;
 
-  // set 0. to double field explicitly just in case
-  tls.last_write_idle = 0.;
-
   if (ssl) {
     set_ssl(ssl);
   }
@@ -124,7 +113,7 @@ void Connection::disconnect() {
 
     tls.wbuf.reset();
     tls.rbuf.reset();
-    tls.last_write_idle = 0.;
+    tls.last_write_idle = {};
     tls.warmup_writelen = 0;
     tls.last_writelen = 0;
     tls.last_readlen = 0;
@@ -266,14 +255,8 @@ long shrpx_bio_ctrl(BIO *b, int cmd, long num, void *ptr) {
 
 namespace {
 int shrpx_bio_create(BIO *b) {
-#if OPENSSL_1_1_API || LIBRESSL_3_5_API
   BIO_set_init(b, 1);
-#else  // !OPENSSL_1_1_API && !LIBRESSL_3_5_API
-  b->init = 1;
-  b->num = 0;
-  b->ptr = nullptr;
-  b->flags = 0;
-#endif // !OPENSSL_1_1_API && !LIBRESSL_3_5_API
+
   return 1;
 }
 } // namespace
@@ -284,18 +267,10 @@ int shrpx_bio_destroy(BIO *b) {
     return 0;
   }
 
-#if !OPENSSL_1_1_API && !LIBRESSL_3_5_API
-  b->ptr = nullptr;
-  b->init = 0;
-  b->flags = 0;
-#endif // !OPENSSL_1_1_API && !LIBRESSL_3_5_API
-
   return 1;
 }
 } // namespace
 
-#if OPENSSL_1_1_API || LIBRESSL_3_5_API
-
 BIO_METHOD *create_bio_method() {
   auto meth = BIO_meth_new(BIO_TYPE_FD, "nghttpx-bio");
   BIO_meth_set_write(meth, shrpx_bio_write);
@@ -309,20 +284,6 @@ BIO_METHOD *create_bio_method() {
   return meth;
 }
 
-#else // !OPENSSL_1_1_API && !LIBRESSL_3_5_API
-
-BIO_METHOD *create_bio_method() {
-  static auto meth = new BIO_METHOD{
-      BIO_TYPE_FD,    "nghttpx-bio",    shrpx_bio_write,
-      shrpx_bio_read, shrpx_bio_puts,   shrpx_bio_gets,
-      shrpx_bio_ctrl, shrpx_bio_create, shrpx_bio_destroy,
-  };
-
-  return meth;
-}
-
-#endif // !OPENSSL_1_1_API && !LIBRESSL_3_5_API
-
 void Connection::set_ssl(SSL *ssl) {
   tls.ssl = ssl;
 
@@ -410,7 +371,7 @@ int Connection::tls_handshake() {
 
   ERR_clear_error();
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#ifdef NGHTTP2_GENUINE_OPENSSL
   if (!tls.server_handshake || tls.early_data_finish) {
     rv = SSL_do_handshake(tls.ssl);
   } else {
@@ -461,9 +422,9 @@ int Connection::tls_handshake() {
       }
     }
   }
-#else  // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#else  // !NGHTTP2_GENUINE_OPENSSL
   rv = SSL_do_handshake(tls.ssl);
-#endif // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#endif // !NGHTTP2_GENUINE_OPENSSL
 
   if (rv <= 0) {
     auto err = SSL_get_error(tls.ssl, rv);
@@ -512,9 +473,9 @@ int Connection::tls_handshake() {
   // routine.  We have to check HTTP/2 requirement if HTTP/2 was
   // negotiated before sending finished message to the peer.
   if ((rv != 1
-#ifdef OPENSSL_IS_BORINGSSL
+#ifdef NGHTTP2_OPENSSL_IS_BORINGSSL
        || SSL_in_init(tls.ssl)
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
            ) &&
       tls.wbuf.rleft()) {
     // First write indicates that resumption stuff has done.
@@ -552,7 +513,7 @@ int Connection::tls_handshake() {
     return SHRPX_ERR_INPROGRESS;
   }
 
-#ifdef OPENSSL_IS_BORINGSSL
+#ifdef NGHTTP2_OPENSSL_IS_BORINGSSL
   if (!tlsconf.no_postpone_early_data && SSL_in_early_data(tls.ssl) &&
       SSL_in_init(tls.ssl)) {
     auto nread = SSL_read(tls.ssl, buf.data(), buf.size());
@@ -584,7 +545,7 @@ int Connection::tls_handshake() {
       return SHRPX_ERR_INPROGRESS;
     }
   }
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
   // Handshake was done
 
@@ -614,14 +575,14 @@ int Connection::tls_handshake_simple() {
   }
 
   int rv;
-#if OPENSSL_1_1_1_API || defined(OPENSSL_IS_BORINGSSL)
+#if defined(NGHTTP2_GENUINE_OPENSSL) || defined(NGHTTP2_OPENSSL_IS_BORINGSSL)
   auto &tlsconf = get_config()->tls;
   std::array<uint8_t, 16_k> buf;
-#endif // OPENSSL_1_1_1_API || defined(OPENSSL_IS_BORINGSSL)
+#endif // NGHTTP2_GENUINE_OPENSSL || NGHTTP2_OPENSSL_IS_BORINGSSL
 
   ERR_clear_error();
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#ifdef NGHTTP2_GENUINE_OPENSSL
   if (!tls.server_handshake || tls.early_data_finish) {
     rv = SSL_do_handshake(tls.ssl);
   } else {
@@ -666,9 +627,9 @@ int Connection::tls_handshake_simple() {
       }
     }
   }
-#else  // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#else  // !NGHTTP2_GENUINE_OPENSSL
   rv = SSL_do_handshake(tls.ssl);
-#endif // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#endif // !NGHTTP2_GENUINE_OPENSSL
 
   if (rv <= 0) {
     auto err = SSL_get_error(tls.ssl, rv);
@@ -707,7 +668,7 @@ int Connection::tls_handshake_simple() {
     return SHRPX_ERR_INPROGRESS;
   }
 
-#ifdef OPENSSL_IS_BORINGSSL
+#ifdef NGHTTP2_OPENSSL_IS_BORINGSSL
   if (!tlsconf.no_postpone_early_data && SSL_in_early_data(tls.ssl) &&
       SSL_in_init(tls.ssl)) {
     auto nread = SSL_read(tls.ssl, buf.data(), buf.size());
@@ -739,7 +700,7 @@ int Connection::tls_handshake_simple() {
       return SHRPX_ERR_INPROGRESS;
     }
   }
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
   // Handshake was done
 
@@ -774,7 +735,7 @@ int Connection::write_tls_pending_handshake() {
     tls.wbuf.drain(nwrite);
   }
 
-#ifdef OPENSSL_IS_BORINGSSL
+#ifdef NGHTTP2_OPENSSL_IS_BORINGSSL
   if (!SSL_in_init(tls.ssl)) {
     // This will send a session ticket.
     auto nwrite = SSL_write(tls.ssl, "", 0);
@@ -802,7 +763,7 @@ int Connection::write_tls_pending_handshake() {
       }
     }
   }
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
   // We have to start read watcher, since later stage of code expects
   // this.
@@ -833,14 +794,7 @@ int Connection::check_http2_requirement() {
   const unsigned char *next_proto = nullptr;
   unsigned int next_proto_len;
 
-#ifndef OPENSSL_NO_NEXTPROTONEG
-  SSL_get0_next_proto_negotiated(tls.ssl, &next_proto, &next_proto_len);
-#endif // !OPENSSL_NO_NEXTPROTONEG
-#if OPENSSL_VERSION_NUMBER >= 0x10002000L
-  if (next_proto == nullptr) {
-    SSL_get0_alpn_selected(tls.ssl, &next_proto, &next_proto_len);
-  }
-#endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
+  SSL_get0_alpn_selected(tls.ssl, &next_proto, &next_proto_len);
   if (next_proto == nullptr ||
       !util::check_h2_is_selected(StringRef{next_proto, next_proto_len})) {
     return 0;
@@ -881,9 +835,9 @@ size_t Connection::get_tls_write_limit() {
     return std::numeric_limits<ssize_t>::max();
   }
 
-  auto t = ev_now(loop);
+  auto t = std::chrono::steady_clock::now();
 
-  if (tls.last_write_idle >= 0. &&
+  if (tls.last_write_idle.time_since_epoch().count() >= 0 &&
       t - tls.last_write_idle > tls_dyn_rec_idle_timeout) {
     // Time out, use small record size
     tls.warmup_writelen = 0;
@@ -904,8 +858,8 @@ void Connection::update_tls_warmup_writelen(size_t n) {
 }
 
 void Connection::start_tls_write_idle() {
-  if (tls.last_write_idle < 0.) {
-    tls.last_write_idle = ev_now(loop);
+  if (tls.last_write_idle.time_since_epoch().count() < 0) {
+    tls.last_write_idle = std::chrono::steady_clock::now();
   }
 }
 
@@ -927,7 +881,7 @@ ssize_t Connection::write_tls(const void *data, size_t len) {
     tls.last_writelen = 0;
   }
 
-  tls.last_write_idle = -1.;
+  tls.last_write_idle = std::chrono::steady_clock::time_point(-1s);
 
   auto &tlsconf = get_config()->tls;
   auto via_bio =
@@ -935,7 +889,7 @@ ssize_t Connection::write_tls(const void *data, size_t len) {
 
   ERR_clear_error();
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#ifdef NGHTTP2_GENUINE_OPENSSL
   int rv;
   if (SSL_is_init_finished(tls.ssl)) {
     rv = SSL_write(tls.ssl, data, len);
@@ -947,9 +901,9 @@ ssize_t Connection::write_tls(const void *data, size_t len) {
       rv = nwrite;
     }
   }
-#else  // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#else  // !NGHTTP2_GENUINE_OPENSSL
   auto rv = SSL_write(tls.ssl, data, len);
-#endif // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#endif // !NGHTTP2_GENUINE_OPENSSL
 
   if (rv <= 0) {
     auto err = SSL_get_error(tls.ssl, rv);
@@ -999,11 +953,11 @@ ssize_t Connection::write_tls(const void *data, size_t len) {
 ssize_t Connection::read_tls(void *data, size_t len) {
   ERR_clear_error();
 
-#if OPENSSL_1_1_1_API
+#if defined(NGHTTP2_GENUINE_OPENSSL) || defined(NGHTTP2_OPENSSL_IS_BORINGSSL)
   if (tls.earlybuf.rleft()) {
     return tls.earlybuf.remove(data, len);
   }
-#endif // OPENSSL_1_1_1_API
+#endif // NGHTTP2_GENUINE_OPENSSL || NGHTTP2_OPENSSL_IS_BORINGSSL
 
   // SSL_read requires the same arguments (buf pointer and its
   // length) on SSL_ERROR_WANT_READ or SSL_ERROR_WANT_WRITE.
@@ -1022,7 +976,11 @@ ssize_t Connection::read_tls(void *data, size_t len) {
     tls.last_readlen = 0;
   }
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+  auto &tlsconf = get_config()->tls;
+  auto via_bio =
+      tls.server_handshake && !tlsconf.session_cache.memcached.host.empty();
+
+#ifdef NGHTTP2_GENUINE_OPENSSL
   if (!tls.early_data_finish) {
     // TLSv1.3 handshake is still going on.
     size_t nread;
@@ -1059,9 +1017,14 @@ ssize_t Connection::read_tls(void *data, size_t len) {
       // We may have stopped write watcher in write_tls.
       wlimit.startw();
     }
+
+    if (!via_bio) {
+      rlimit.drain(nread);
+    }
+
     return nread;
   }
-#endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#endif // NGHTTP2_GENUINE_OPENSSL
 
   auto rv = SSL_read(tls.ssl, data, len);
 
@@ -1091,6 +1054,10 @@ ssize_t Connection::read_tls(void *data, size_t len) {
     }
   }
 
+  if (!via_bio) {
+    rlimit.drain(rv);
+  }
+
   return rv;
 }
 
@@ -1285,17 +1252,18 @@ void Connection::again_rt(ev_tstamp t) {
   read_timeout = t;
   rt.repeat = t;
   ev_timer_again(loop, &rt);
-  last_read = ev_now(loop);
+  last_read = std::chrono::steady_clock::now();
 }
 
 void Connection::again_rt() {
   rt.repeat = read_timeout;
   ev_timer_again(loop, &rt);
-  last_read = ev_now(loop);
+  last_read = std::chrono::steady_clock::now();
 }
 
 bool Connection::expired_rt() {
-  auto delta = read_timeout - (ev_now(loop) - last_read);
+  auto delta = read_timeout - util::ev_tstamp_from(
+                                  std::chrono::steady_clock::now() - last_read);
   if (delta < 1e-9) {
     return true;
   }
