@@ -1672,14 +1672,7 @@ int Http2Session::connection_made() {
     const unsigned char *next_proto = nullptr;
     unsigned int next_proto_len = 0;
 
-#ifndef OPENSSL_NO_NEXTPROTONEG
-    SSL_get0_next_proto_negotiated(conn_.tls.ssl, &next_proto, &next_proto_len);
-#endif // !OPENSSL_NO_NEXTPROTONEG
-#if OPENSSL_VERSION_NUMBER >= 0x10002000L
-    if (!next_proto) {
-      SSL_get0_alpn_selected(conn_.tls.ssl, &next_proto, &next_proto_len);
-    }
-#endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
+    SSL_get0_alpn_selected(conn_.tls.ssl, &next_proto, &next_proto_len);
 
     if (!next_proto) {
       downstream_failure(addr_, raddr_);
@@ -1834,9 +1827,7 @@ void Http2Session::signal_write() {
   }
 }
 
-struct ev_loop *Http2Session::get_loop() const {
-  return conn_.loop;
-}
+struct ev_loop *Http2Session::get_loop() const { return conn_.loop; }
 
 ev_io *Http2Session::get_wev() { return &conn_.wev; }
 
@@ -2018,7 +2009,7 @@ int Http2Session::connected() {
 }
 
 int Http2Session::read_clear() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   std::array<uint8_t, 16_k> buf;
 
@@ -2040,7 +2031,7 @@ int Http2Session::read_clear() {
 }
 
 int Http2Session::write_clear() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   std::array<struct iovec, MAX_WR_IOVCNT> iov;
 
@@ -2081,7 +2072,7 @@ int Http2Session::write_clear() {
 }
 
 int Http2Session::tls_handshake() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   ERR_clear_error();
 
@@ -2120,7 +2111,7 @@ int Http2Session::tls_handshake() {
 }
 
 int Http2Session::read_tls() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   std::array<uint8_t, 16_k> buf;
 
@@ -2144,7 +2135,7 @@ int Http2Session::read_tls() {
 }
 
 int Http2Session::write_tls() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   ERR_clear_error();
 
