@@ -57,21 +57,21 @@ LogConfig::LogConfig()
 }
 
 #ifndef NOTHREADS
-#ifdef HAVE_THREAD_LOCAL
+#  ifdef HAVE_THREAD_LOCAL
 namespace {
-thread_local std::unique_ptr<LogConfig> config = make_unique<LogConfig>();
+thread_local std::unique_ptr<LogConfig> config = std::make_unique<LogConfig>();
 } // namespace
 
 LogConfig *log_config() { return config.get(); }
 void delete_log_config() {}
-#else  // !HAVE_THREAD_LOCAL
+#  else  // !HAVE_THREAD_LOCAL
 namespace {
 pthread_key_t lckey;
 pthread_once_t lckey_once = PTHREAD_ONCE_INIT;
 } // namespace
 
 namespace {
-void make_key() { pthread_key_create(&lckey, NULL); }
+void make_key() { pthread_key_create(&lckey, nullptr); }
 } // namespace
 
 LogConfig *log_config() {
@@ -85,16 +85,16 @@ LogConfig *log_config() {
 }
 
 void delete_log_config() { delete log_config(); }
-#endif // !HAVE_THREAD_LOCAL
-#else  // NOTHREADS
+#  endif // !HAVE_THREAD_LOCAL
+#else    // NOTHREADS
 namespace {
-std::unique_ptr<LogConfig> config = make_unique<LogConfig>();
+std::unique_ptr<LogConfig> config = std::make_unique<LogConfig>();
 } // namespace
 
 LogConfig *log_config() { return config.get(); }
 
 void delete_log_config() {}
-#endif // NOTHREADS
+#endif   // NOTHREADS
 
 void LogConfig::update_tstamp_millis(
     const std::chrono::system_clock::time_point &now) {
