@@ -55,18 +55,18 @@
 #  include <openssl/core_names.h>
 #  include <openssl/decoder.h>
 #endif // OPENSSL_3_0_0_API
-#ifdef OPENSSL_IS_BORINGSSL
+#ifdef NGHTTP2_OPENSSL_IS_BORINGSSL
 #  include <openssl/hmac.h>
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
 #include <nghttp2/nghttp2.h>
 
 #ifdef ENABLE_HTTP3
 #  include <ngtcp2/ngtcp2.h>
 #  include <ngtcp2/ngtcp2_crypto.h>
-#  ifdef HAVE_LIBNGTCP2_CRYPTO_OPENSSL
-#    include <ngtcp2/ngtcp2_crypto_openssl.h>
-#  endif // HAVE_LIBNGTCP2_CRYPTO_OPENSSL
+#  ifdef HAVE_LIBNGTCP2_CRYPTO_QUICTLS
+#    include <ngtcp2/ngtcp2_crypto_quictls.h>
+#  endif // HAVE_LIBNGTCP2_CRYPTO_QUICTLS
 #  ifdef HAVE_LIBNGTCP2_CRYPTO_BORINGSSL
 #    include <ngtcp2/ngtcp2_crypto_boringssl.h>
 #  endif // HAVE_LIBNGTCP2_CRYPTO_BORINGSSL
@@ -90,31 +90,12 @@
 #include "timegm.h"
 
 using namespace nghttp2;
+using namespace std::chrono_literals;
 
 namespace shrpx {
 
 namespace tls {
 
-#if !OPENSSL_1_1_API
-namespace {
-const unsigned char *ASN1_STRING_get0_data(ASN1_STRING *x) {
-  return ASN1_STRING_data(x);
-}
-} // namespace
-#endif // !OPENSSL_1_1_API
-
-#ifndef OPENSSL_NO_NEXTPROTONEG
-namespace {
-int next_proto_cb(SSL *s, const unsigned char **data, unsigned int *len,
-                  void *arg) {
-  auto &prefs = get_config()->tls.alpn_prefs;
-  *data = prefs.data();
-  *len = prefs.size();
-  return SSL_TLSEXT_ERR_OK;
-}
-} // namespace
-#endif // !OPENSSL_NO_NEXTPROTONEG
-
 namespace {
 int verify_callback(int preverify_ok, X509_STORE_CTX *ctx) {
   if (!preverify_ok) {
@@ -230,8 +211,7 @@ int servername_callback(SSL *ssl, int *al, void *arg) {
 
   assert(!ssl_ctx_list.empty());
 
-#if !defined(OPENSSL_IS_BORINGSSL) && !LIBRESSL_IN_USE &&                      \
-    OPENSSL_VERSION_NUMBER >= 0x10002000L
+#ifdef NGHTTP2_GENUINE_OPENSSL
   auto num_sigalgs =
       SSL_get_sigalgs(ssl, 0, nullptr, nullptr, nullptr, nullptr, nullptr);
 
@@ -273,12 +253,7 @@ int servername_callback(SSL *ssl, int *al, void *arg) {
 
     for (auto ssl_ctx : ssl_ctx_list) {
       auto cert = SSL_CTX_get0_certificate(ssl_ctx);
-
-#  if OPENSSL_1_1_API
       auto pubkey = X509_get0_pubkey(cert);
-#  else  // !OPENSSL_1_1_API
-      auto pubkey = X509_get_pubkey(cert);
-#  endif // !OPENSSL_1_1_API
 
       if (EVP_PKEY_base_id(pubkey) != EVP_PKEY_EC) {
         continue;
@@ -295,13 +270,8 @@ int servername_callback(SSL *ssl, int *al, void *arg) {
         SSL_set_SSL_CTX(ssl, ssl_ctx);
         return SSL_TLSEXT_ERR_OK;
       }
-#  else // !OPENSSL_3_0_0_API
-#    if OPENSSL_1_1_API
+#  else  // !OPENSSL_3_0_0_API
       auto eckey = EVP_PKEY_get0_EC_KEY(pubkey);
-#    else  // !OPENSSL_1_1_API
-      auto eckey = EVP_PKEY_get1_EC_KEY(pubkey);
-#    endif // !OPENSSL_1_1_API
-
       if (eckey == nullptr) {
         continue;
       }
@@ -309,20 +279,14 @@ int servername_callback(SSL *ssl, int *al, void *arg) {
       auto ecgroup = EC_KEY_get0_group(eckey);
       auto cert_curve = EC_GROUP_get_curve_name(ecgroup);
 
-#    if !OPENSSL_1_1_API
-      EC_KEY_free(eckey);
-      EVP_PKEY_free(pubkey);
-#    endif // !OPENSSL_1_1_API
-
       if (shared_curve == cert_curve) {
         SSL_set_SSL_CTX(ssl, ssl_ctx);
         return SSL_TLSEXT_ERR_OK;
       }
-#  endif   // !OPENSSL_3_0_0_API
+#  endif // !OPENSSL_3_0_0_API
     }
   }
-#endif // !defined(OPENSSL_IS_BORINGSSL) && !LIBRESSL_IN_USE &&
-       // OPENSSL_VERSION_NUMBER >= 0x10002000L
+#endif // NGHTTP2_GENUINE_OPENSSL
 
   SSL_set_SSL_CTX(ssl, ssl_ctx_list[0]);
 
@@ -330,7 +294,7 @@ int servername_callback(SSL *ssl, int *al, void *arg) {
 }
 } // namespace
 
-#ifndef OPENSSL_IS_BORINGSSL
+#ifndef NGHTTP2_OPENSSL_IS_BORINGSSL
 namespace {
 std::shared_ptr<std::vector<uint8_t>>
 get_ocsp_data(TLSContextData *tls_ctx_data) {
@@ -356,8 +320,8 @@ int ocsp_resp_cb(SSL *ssl, void *arg) {
     return SSL_TLSEXT_ERR_OK;
   }
 
-  auto buf =
-      static_cast<uint8_t *>(CRYPTO_malloc(data->size(), __FILE__, __LINE__));
+  auto buf = static_cast<uint8_t *>(
+      CRYPTO_malloc(data->size(), NGHTTP2_FILE_NAME, __LINE__));
 
   if (!buf) {
     return SSL_TLSEXT_ERR_OK;
@@ -370,7 +334,7 @@ int ocsp_resp_cb(SSL *ssl, void *arg) {
   return SSL_TLSEXT_ERR_OK;
 }
 } // namespace
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
 constexpr auto MEMCACHED_SESSION_CACHE_KEY_PREFIX =
     StringRef::from_lit("nghttpx:tls-session-cache:");
@@ -383,7 +347,7 @@ int tls_session_client_new_cb(SSL *ssl, SSL_SESSION *session) {
   }
 
   try_cache_tls_session(conn->tls.client_session_cache, session,
-                        ev_now(conn->loop));
+                        std::chrono::steady_clock::now());
 
   return 0;
 }
@@ -446,13 +410,8 @@ int tls_session_new_cb(SSL *ssl, SSL_SESSION *session) {
 } // namespace
 
 namespace {
-SSL_SESSION *tls_session_get_cb(SSL *ssl,
-#if OPENSSL_1_1_API || LIBRESSL_2_7_API
-                                const unsigned char *id,
-#else  // !(OPENSSL_1_1_API || LIBRESSL_2_7_API)
-                                unsigned char *id,
-#endif // !(OPENSSL_1_1_API || LIBRESSL_2_7_API)
-                                int idlen, int *copy) {
+SSL_SESSION *tls_session_get_cb(SSL *ssl, const unsigned char *id, int idlen,
+                                int *copy) {
   auto conn = static_cast<Connection *>(SSL_get_app_data(ssl));
   auto handler = static_cast<ClientHandler *>(conn->data);
   auto worker = handler->get_worker();
@@ -675,15 +634,14 @@ void info_callback(const SSL *ssl, int where, int ret) {
 }
 } // namespace
 
-#if OPENSSL_VERSION_NUMBER >= 0x10002000L
 namespace {
 int alpn_select_proto_cb(SSL *ssl, const unsigned char **out,
                          unsigned char *outlen, const unsigned char *in,
                          unsigned int inlen, void *arg) {
-  // We assume that get_config()->npn_list contains ALPN protocol
+  // We assume that get_config()->alpn_list contains ALPN protocol
   // identifier sorted by preference order.  So we just break when we
   // found the first overlap.
-  for (const auto &target_proto_id : get_config()->tls.npn_list) {
+  for (const auto &target_proto_id : get_config()->tls.alpn_list) {
     for (auto p = in, end = in + inlen; p < end;) {
       auto proto_id = p + 1;
       auto proto_len = *p;
@@ -704,10 +662,8 @@ int alpn_select_proto_cb(SSL *ssl, const unsigned char **out,
   return SSL_TLSEXT_ERR_NOACK;
 }
 } // namespace
-#endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
 
 #ifdef ENABLE_HTTP3
-#  if OPENSSL_VERSION_NUMBER >= 0x10002000L
 namespace {
 int quic_alpn_select_proto_cb(SSL *ssl, const unsigned char **out,
                               unsigned char *outlen, const unsigned char *in,
@@ -737,16 +693,9 @@ int quic_alpn_select_proto_cb(SSL *ssl, const unsigned char **out,
   return SSL_TLSEXT_ERR_ALERT_FATAL;
 }
 } // namespace
-#  endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
-#endif   // ENABLE_HTTP3
-
-#if !LIBRESSL_IN_USE && OPENSSL_VERSION_NUMBER >= 0x10002000L &&               \
-    !defined(OPENSSL_IS_BORINGSSL)
-
-#  ifndef TLSEXT_TYPE_signed_certificate_timestamp
-#    define TLSEXT_TYPE_signed_certificate_timestamp 18
-#  endif // !TLSEXT_TYPE_signed_certificate_timestamp
+#endif // ENABLE_HTTP3
 
+#ifdef NGHTTP2_GENUINE_OPENSSL
 namespace {
 int sct_add_cb(SSL *ssl, unsigned int ext_type, unsigned int context,
                const unsigned char **out, size_t *outlen, X509 *x,
@@ -805,34 +754,7 @@ int sct_parse_cb(SSL *ssl, unsigned int ext_type, unsigned int context,
 }
 } // namespace
 
-#  if !OPENSSL_1_1_1_API
-
-namespace {
-int legacy_sct_add_cb(SSL *ssl, unsigned int ext_type,
-                      const unsigned char **out, size_t *outlen, int *al,
-                      void *add_arg) {
-  return sct_add_cb(ssl, ext_type, 0, out, outlen, nullptr, 0, al, add_arg);
-}
-} // namespace
-
-namespace {
-void legacy_sct_free_cb(SSL *ssl, unsigned int ext_type,
-                        const unsigned char *out, void *add_arg) {
-  sct_free_cb(ssl, ext_type, 0, out, add_arg);
-}
-} // namespace
-
-namespace {
-int legacy_sct_parse_cb(SSL *ssl, unsigned int ext_type,
-                        const unsigned char *in, size_t inlen, int *al,
-                        void *parse_arg) {
-  return sct_parse_cb(ssl, ext_type, 0, in, inlen, nullptr, 0, al, parse_arg);
-}
-} // namespace
-
-#  endif // !OPENSSL_1_1_1_API
-#endif   // !LIBRESSL_IN_USE && OPENSSL_VERSION_NUMBER >= 0x10002000L &&
-         // !defined(OPENSSL_IS_BORINGSSL)
+#endif // NGHTTP2_GENUINE_OPENSSL
 
 #ifndef OPENSSL_NO_PSK
 namespace {
@@ -941,7 +863,7 @@ SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file,
                   SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION |
                   SSL_OP_SINGLE_ECDH_USE | SSL_OP_SINGLE_DH_USE |
                   SSL_OP_CIPHER_SERVER_PREFERENCE
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#ifdef NGHTTP2_GENUINE_OPENSSL
                   // The reason for disabling built-in anti-replay in
                   // OpenSSL is that it only works if client gets back
                   // to the same server.  The freshness check
@@ -949,7 +871,7 @@ SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file,
                   // https://tools.ietf.org/html/rfc8446#section-8.3
                   // is still performed.
                   | SSL_OP_NO_ANTI_REPLAY
-#endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#endif // NGHTTP2_GENUINE_OPENSSL
       ;
 
   auto config = mod_config();
@@ -986,39 +908,21 @@ SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file,
     DIE();
   }
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#if defined(NGHTTP2_GENUINE_OPENSSL) || defined(NGHTTP2_OPENSSL_IS_LIBRESSL)
   if (SSL_CTX_set_ciphersuites(ssl_ctx, tlsconf.tls13_ciphers.c_str()) == 0) {
     LOG(FATAL) << "SSL_CTX_set_ciphersuites " << tlsconf.tls13_ciphers
                << " failed: " << ERR_error_string(ERR_get_error(), nullptr);
     DIE();
   }
-#endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#endif // NGHTTP2_GENUINE_OPENSSL || NGHTTP2_OPENSSL_IS_LIBRESSL
 
 #ifndef OPENSSL_NO_EC
-#  if !LIBRESSL_LEGACY_API && OPENSSL_VERSION_NUMBER >= 0x10002000L
   if (SSL_CTX_set1_curves_list(ssl_ctx, tlsconf.ecdh_curves.c_str()) != 1) {
     LOG(FATAL) << "SSL_CTX_set1_curves_list " << tlsconf.ecdh_curves
                << " failed";
     DIE();
   }
-#    if !defined(OPENSSL_IS_BORINGSSL) && !OPENSSL_1_1_API
-  // It looks like we need this function call for OpenSSL 1.0.2.  This
-  // function was deprecated in OpenSSL 1.1.0 and BoringSSL.
-  SSL_CTX_set_ecdh_auto(ssl_ctx, 1);
-#    endif // !defined(OPENSSL_IS_BORINGSSL) && !OPENSSL_1_1_API
-#  else    // LIBRESSL_LEGACY_API || OPENSSL_VERSION_NUBMER < 0x10002000L
-  // Use P-256, which is sufficiently secure at the time of this
-  // writing.
-  auto ecdh = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
-  if (ecdh == nullptr) {
-    LOG(FATAL) << "EC_KEY_new_by_curv_name failed: "
-               << ERR_error_string(ERR_get_error(), nullptr);
-    DIE();
-  }
-  SSL_CTX_set_tmp_ecdh(ssl_ctx, ecdh);
-  EC_KEY_free(ecdh);
-#  endif   // LIBRESSL_LEGACY_API || OPENSSL_VERSION_NUBMER < 0x10002000L
-#endif     // OPENSSL_NO_EC
+#endif // OPENSSL_NO_EC
 
   if (!tlsconf.dh_param_file.empty()) {
     // Read DH parameters from file
@@ -1140,23 +1044,17 @@ SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file,
 #else  // !OPENSSL_3_0_0_API
   SSL_CTX_set_tlsext_ticket_key_cb(ssl_ctx, ticket_key_cb);
 #endif // !OPENSSL_3_0_0_API
-#ifndef OPENSSL_IS_BORINGSSL
+#ifndef NGHTTP2_OPENSSL_IS_BORINGSSL
   SSL_CTX_set_tlsext_status_cb(ssl_ctx, ocsp_resp_cb);
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
   SSL_CTX_set_info_callback(ssl_ctx, info_callback);
 
-#ifdef OPENSSL_IS_BORINGSSL
+#ifdef NGHTTP2_OPENSSL_IS_BORINGSSL
   SSL_CTX_set_early_data_enabled(ssl_ctx, 1);
-#endif // OPENSSL_IS_BORINGSSL
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
-  // NPN advertisement
-#ifndef OPENSSL_NO_NEXTPROTONEG
-  SSL_CTX_set_next_protos_advertised_cb(ssl_ctx, next_proto_cb, nullptr);
-#endif // !OPENSSL_NO_NEXTPROTONEG
-#if OPENSSL_VERSION_NUMBER >= 0x10002000L
   // ALPN selection callback
   SSL_CTX_set_alpn_select_cb(ssl_ctx, alpn_select_proto_cb, nullptr);
-#endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
 
   auto tls_ctx_data = new TLSContextData();
   tls_ctx_data->cert_file = cert_file;
@@ -1164,14 +1062,12 @@ SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file,
 
   SSL_CTX_set_app_data(ssl_ctx, tls_ctx_data);
 
-#if !LIBRESSL_IN_USE && OPENSSL_VERSION_NUMBER >= 0x10002000L &&               \
-    !defined(OPENSSL_IS_BORINGSSL)
+#ifdef NGHTTP2_GENUINE_OPENSSL
   // SSL_extension_supported(TLSEXT_TYPE_signed_certificate_timestamp)
   // returns 1, which means OpenSSL internally handles it.  But
   // OpenSSL handles signed_certificate_timestamp extension specially,
   // and it lets custom handler to process the extension.
   if (!sct_data.empty()) {
-#  if OPENSSL_1_1_1_API
     // It is not entirely clear to me that SSL_EXT_CLIENT_HELLO is
     // required here.  sct_parse_cb is called without
     // SSL_EXT_CLIENT_HELLO being set.  But the passed context value
@@ -1185,18 +1081,8 @@ SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file,
                  << ERR_error_string(ERR_get_error(), nullptr);
       DIE();
     }
-#  else  // !OPENSSL_1_1_1_API
-    if (SSL_CTX_add_server_custom_ext(
-            ssl_ctx, TLSEXT_TYPE_signed_certificate_timestamp,
-            legacy_sct_add_cb, legacy_sct_free_cb, nullptr, legacy_sct_parse_cb,
-            nullptr) != 1) {
-      LOG(FATAL) << "SSL_CTX_add_server_custom_ext failed: "
-                 << ERR_error_string(ERR_get_error(), nullptr);
-      DIE();
-    }
-#  endif // !OPENSSL_1_1_1_API
   }
-#elif defined(OPENSSL_IS_BORINGSSL)
+#elif defined(NGHTTP2_OPENSSL_IS_BORINGSSL)
   if (!tls_ctx_data->sct_data.empty() &&
       SSL_CTX_set_signed_cert_timestamp_list(
           ssl_ctx, tls_ctx_data->sct_data.data(),
@@ -1205,15 +1091,20 @@ SSL_CTX *create_ssl_context(const char *private_key_file, const char *cert_file,
                << ERR_error_string(ERR_get_error(), nullptr);
     DIE();
   }
-#endif // defined(OPENSSL_IS_BORINGSSL)
+#endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#ifdef NGHTTP2_GENUINE_OPENSSL
   if (SSL_CTX_set_max_early_data(ssl_ctx, tlsconf.max_early_data) != 1) {
     LOG(FATAL) << "SSL_CTX_set_max_early_data failed: "
                << ERR_error_string(ERR_get_error(), nullptr);
     DIE();
   }
-#endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+  if (SSL_CTX_set_recv_max_early_data(ssl_ctx, tlsconf.max_early_data) != 1) {
+    LOG(FATAL) << "SSL_CTX_set_recv_max_early_data failed: "
+               << ERR_error_string(ERR_get_error(), nullptr);
+    DIE();
+  }
+#endif // NGHTTP2_GENUINE_OPENSSL
 
 #ifndef OPENSSL_NO_PSK
   SSL_CTX_set_psk_server_callback(ssl_ctx, psk_server_cb);
@@ -1242,14 +1133,14 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
       SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION | SSL_OP_SINGLE_ECDH_USE |
       SSL_OP_SINGLE_DH_USE |
       SSL_OP_CIPHER_SERVER_PREFERENCE
-#  if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#  ifdef NGHTTP2_GENUINE_OPENSSL
       // The reason for disabling built-in anti-replay in OpenSSL is
       // that it only works if client gets back to the same server.
       // The freshness check described in
       // https://tools.ietf.org/html/rfc8446#section-8.3 is still
       // performed.
       | SSL_OP_NO_ANTI_REPLAY
-#  endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#  endif // NGHTTP2_GENUINE_OPENSSL
       ;
 
   auto config = mod_config();
@@ -1257,12 +1148,12 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
 
   SSL_CTX_set_options(ssl_ctx, ssl_opts);
 
-#  ifdef HAVE_LIBNGTCP2_CRYPTO_OPENSSL
-  if (ngtcp2_crypto_openssl_configure_server_context(ssl_ctx) != 0) {
-    LOG(FATAL) << "ngtcp2_crypto_openssl_configure_server_context failed";
+#  ifdef HAVE_LIBNGTCP2_CRYPTO_QUICTLS
+  if (ngtcp2_crypto_quictls_configure_server_context(ssl_ctx) != 0) {
+    LOG(FATAL) << "ngtcp2_crypto_quictls_configure_server_context failed";
     DIE();
   }
-#  endif // HAVE_LIBNGTCP2_CRYPTO_OPENSSL
+#  endif // HAVE_LIBNGTCP2_CRYPTO_QUICTLS
 #  ifdef HAVE_LIBNGTCP2_CRYPTO_BORINGSSL
   if (ngtcp2_crypto_boringssl_configure_server_context(ssl_ctx) != 0) {
     LOG(FATAL) << "ngtcp2_crypto_boringssl_configure_server_context failed";
@@ -1282,39 +1173,21 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
     DIE();
   }
 
-#  if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#  if defined(NGHTTP2_GENUINE_OPENSSL) || defined(NGHTTP2_OPENSSL_IS_LIBRESSL)
   if (SSL_CTX_set_ciphersuites(ssl_ctx, tlsconf.tls13_ciphers.c_str()) == 0) {
     LOG(FATAL) << "SSL_CTX_set_ciphersuites " << tlsconf.tls13_ciphers
                << " failed: " << ERR_error_string(ERR_get_error(), nullptr);
     DIE();
   }
-#  endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#  endif // NGHTTP2_GENUINE_OPENSSL || NGHTTP2_OPENSSL_IS_LIBRESSL
 
 #  ifndef OPENSSL_NO_EC
-#    if !LIBRESSL_LEGACY_API && OPENSSL_VERSION_NUMBER >= 0x10002000L
   if (SSL_CTX_set1_curves_list(ssl_ctx, tlsconf.ecdh_curves.c_str()) != 1) {
     LOG(FATAL) << "SSL_CTX_set1_curves_list " << tlsconf.ecdh_curves
                << " failed";
     DIE();
   }
-#      if !defined(OPENSSL_IS_BORINGSSL) && !OPENSSL_1_1_API
-  // It looks like we need this function call for OpenSSL 1.0.2.  This
-  // function was deprecated in OpenSSL 1.1.0 and BoringSSL.
-  SSL_CTX_set_ecdh_auto(ssl_ctx, 1);
-#      endif // !defined(OPENSSL_IS_BORINGSSL) && !OPENSSL_1_1_API
-#    else    // LIBRESSL_LEGACY_API || OPENSSL_VERSION_NUBMER < 0x10002000L
-  // Use P-256, which is sufficiently secure at the time of this
-  // writing.
-  auto ecdh = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
-  if (ecdh == nullptr) {
-    LOG(FATAL) << "EC_KEY_new_by_curv_name failed: "
-               << ERR_error_string(ERR_get_error(), nullptr);
-    DIE();
-  }
-  SSL_CTX_set_tmp_ecdh(ssl_ctx, ecdh);
-  EC_KEY_free(ecdh);
-#    endif   // LIBRESSL_LEGACY_API || OPENSSL_VERSION_NUBMER < 0x10002000L
-#  endif     // OPENSSL_NO_EC
+#  endif // OPENSSL_NO_EC
 
   if (!tlsconf.dh_param_file.empty()) {
     // Read DH parameters from file
@@ -1436,14 +1309,12 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
 #  else  // !OPENSSL_3_0_0_API
   SSL_CTX_set_tlsext_ticket_key_cb(ssl_ctx, ticket_key_cb);
 #  endif // !OPENSSL_3_0_0_API
-#  ifndef OPENSSL_IS_BORINGSSL
+#  ifndef NGHTTP2_OPENSSL_IS_BORINGSSL
   SSL_CTX_set_tlsext_status_cb(ssl_ctx, ocsp_resp_cb);
-#  endif // OPENSSL_IS_BORINGSSL
+#  endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
-#  if OPENSSL_VERSION_NUMBER >= 0x10002000L
   // ALPN selection callback
   SSL_CTX_set_alpn_select_cb(ssl_ctx, quic_alpn_select_proto_cb, nullptr);
-#  endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
 
   auto tls_ctx_data = new TLSContextData();
   tls_ctx_data->cert_file = cert_file;
@@ -1451,14 +1322,12 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
 
   SSL_CTX_set_app_data(ssl_ctx, tls_ctx_data);
 
-#  if !LIBRESSL_IN_USE && OPENSSL_VERSION_NUMBER >= 0x10002000L &&             \
-      !defined(OPENSSL_IS_BORINGSSL)
+#  ifdef NGHTTP2_GENUINE_OPENSSL
   // SSL_extension_supported(TLSEXT_TYPE_signed_certificate_timestamp)
   // returns 1, which means OpenSSL internally handles it.  But
   // OpenSSL handles signed_certificate_timestamp extension specially,
   // and it lets custom handler to process the extension.
   if (!sct_data.empty()) {
-#    if OPENSSL_1_1_1_API
     // It is not entirely clear to me that SSL_EXT_CLIENT_HELLO is
     // required here.  sct_parse_cb is called without
     // SSL_EXT_CLIENT_HELLO being set.  But the passed context value
@@ -1472,18 +1341,8 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
                  << ERR_error_string(ERR_get_error(), nullptr);
       DIE();
     }
-#    else  // !OPENSSL_1_1_1_API
-    if (SSL_CTX_add_server_custom_ext(
-            ssl_ctx, TLSEXT_TYPE_signed_certificate_timestamp,
-            legacy_sct_add_cb, legacy_sct_free_cb, nullptr, legacy_sct_parse_cb,
-            nullptr) != 1) {
-      LOG(FATAL) << "SSL_CTX_add_server_custom_ext failed: "
-                 << ERR_error_string(ERR_get_error(), nullptr);
-      DIE();
-    }
-#    endif // !OPENSSL_1_1_1_API
   }
-#  elif defined(OPENSSL_IS_BORINGSSL)
+#  elif defined(NGHTTP2_OPENSSL_IS_BORINGSSL)
   if (!tls_ctx_data->sct_data.empty() &&
       SSL_CTX_set_signed_cert_timestamp_list(
           ssl_ctx, tls_ctx_data->sct_data.data(),
@@ -1492,9 +1351,9 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
                << ERR_error_string(ERR_get_error(), nullptr);
     DIE();
   }
-#  endif // defined(OPENSSL_IS_BORINGSSL)
+#  endif // NGHTTP2_OPENSSL_IS_BORINGSSL
 
-#  if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#  ifdef NGHTTP2_GENUINE_OPENSSL
   auto &quicconf = config->quic;
 
   if (quicconf.upstream.early_data &&
@@ -1504,7 +1363,7 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
                << ERR_error_string(ERR_get_error(), nullptr);
     DIE();
   }
-#  endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#  endif // NGHTTP2_GENUINE_OPENSSL
 
 #  ifndef OPENSSL_NO_PSK
   SSL_CTX_set_psk_server_callback(ssl_ctx, psk_server_cb);
@@ -1514,62 +1373,12 @@ SSL_CTX *create_quic_ssl_context(const char *private_key_file,
 }
 #endif // ENABLE_HTTP3
 
-namespace {
-int select_h2_next_proto_cb(SSL *ssl, unsigned char **out,
-                            unsigned char *outlen, const unsigned char *in,
-                            unsigned int inlen, void *arg) {
-  if (!util::select_h2(const_cast<const unsigned char **>(out), outlen, in,
-                       inlen)) {
-    return SSL_TLSEXT_ERR_NOACK;
-  }
-
-  return SSL_TLSEXT_ERR_OK;
-}
-} // namespace
-
-namespace {
-int select_h1_next_proto_cb(SSL *ssl, unsigned char **out,
-                            unsigned char *outlen, const unsigned char *in,
-                            unsigned int inlen, void *arg) {
-  auto end = in + inlen;
-  for (; in < end;) {
-    if (util::streq(NGHTTP2_H1_1_ALPN, StringRef{in, in + (in[0] + 1)})) {
-      *out = const_cast<unsigned char *>(in) + 1;
-      *outlen = in[0];
-      return SSL_TLSEXT_ERR_OK;
-    }
-    in += in[0] + 1;
-  }
-
-  return SSL_TLSEXT_ERR_NOACK;
-}
-} // namespace
-
-namespace {
-int select_next_proto_cb(SSL *ssl, unsigned char **out, unsigned char *outlen,
-                         const unsigned char *in, unsigned int inlen,
-                         void *arg) {
-  auto conn = static_cast<Connection *>(SSL_get_app_data(ssl));
-  switch (conn->proto) {
-  case Proto::HTTP1:
-    return select_h1_next_proto_cb(ssl, out, outlen, in, inlen, arg);
-  case Proto::HTTP2:
-    return select_h2_next_proto_cb(ssl, out, outlen, in, inlen, arg);
-  default:
-    return SSL_TLSEXT_ERR_NOACK;
-  }
-}
-} // namespace
-
 SSL_CTX *create_ssl_client_context(
 #ifdef HAVE_NEVERBLEED
     neverbleed_t *nb,
 #endif // HAVE_NEVERBLEED
     const StringRef &cacert, const StringRef &cert_file,
-    const StringRef &private_key_file,
-    int (*next_proto_select_cb)(SSL *s, unsigned char **out,
-                                unsigned char *outlen, const unsigned char *in,
-                                unsigned int inlen, void *arg)) {
+    const StringRef &private_key_file) {
   auto ssl_ctx = SSL_CTX_new(TLS_client_method());
   if (!ssl_ctx) {
     LOG(FATAL) << ERR_error_string(ERR_get_error(), nullptr);
@@ -1606,14 +1415,14 @@ SSL_CTX *create_ssl_client_context(
     DIE();
   }
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#if defined(NGHTTP2_GENUINE_OPENSSL) || defined(NGHTTP2_OPENSSL_IS_LIBRESSL)
   if (SSL_CTX_set_ciphersuites(ssl_ctx, tlsconf.client.tls13_ciphers.c_str()) ==
       0) {
     LOG(FATAL) << "SSL_CTX_set_ciphersuites " << tlsconf.client.tls13_ciphers
                << " failed: " << ERR_error_string(ERR_get_error(), nullptr);
     DIE();
   }
-#endif // OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#endif // NGHTTP2_GENUINE_OPENSSL || NGHTTP2_OPENSSL_IS_LIBRESSL
 
   SSL_CTX_set_mode(ssl_ctx, SSL_MODE_RELEASE_BUFFERS);
 
@@ -1669,12 +1478,6 @@ SSL_CTX *create_ssl_client_context(
   SSL_CTX_set_psk_client_callback(ssl_ctx, psk_client_cb);
 #endif // !OPENSSL_NO_PSK
 
-  // NPN selection callback.  This is required to set SSL_CTX because
-  // OpenSSL does not offer SSL_set_next_proto_select_cb.
-#ifndef OPENSSL_NO_NEXTPROTONEG
-  SSL_CTX_set_next_proto_select_cb(ssl_ctx, next_proto_select_cb, nullptr);
-#endif // !OPENSSL_NO_NEXTPROTONEG
-
   return ssl_ctx;
 }
 
@@ -1813,16 +1616,18 @@ StringRef get_common_name(X509 *cert) {
 }
 } // namespace
 
-namespace {
 int verify_numeric_hostname(X509 *cert, const StringRef &hostname,
                             const Address *addr) {
   const void *saddr;
+  size_t saddrlen;
   switch (addr->su.storage.ss_family) {
   case AF_INET:
     saddr = &addr->su.in.sin_addr;
+    saddrlen = sizeof(addr->su.in.sin_addr);
     break;
   case AF_INET6:
     saddr = &addr->su.in6.sin6_addr;
+    saddrlen = sizeof(addr->su.in6.sin6_addr);
     break;
   default:
     return -1;
@@ -1847,7 +1652,7 @@ int verify_numeric_hostname(X509 *cert, const StringRef &hostname,
       size_t ip_addrlen = altname->d.iPAddress->length;
 
       ip_found = true;
-      if (addr->len == ip_addrlen && memcmp(saddr, ip_addr, ip_addrlen) == 0) {
+      if (saddrlen == ip_addrlen && memcmp(saddr, ip_addr, ip_addrlen) == 0) {
         return 0;
       }
     }
@@ -1872,15 +1677,8 @@ int verify_numeric_hostname(X509 *cert, const StringRef &hostname,
 
   return -1;
 }
-} // namespace
-
-namespace {
-int verify_hostname(X509 *cert, const StringRef &hostname,
-                    const Address *addr) {
-  if (util::numeric_host(hostname.c_str())) {
-    return verify_numeric_hostname(cert, hostname, addr);
-  }
 
+int verify_dns_hostname(X509 *cert, const StringRef &hostname) {
   auto altnames = static_cast<GENERAL_NAMES *>(
       X509_get_ext_d2i(cert, NID_subject_alt_name, nullptr, nullptr));
   if (altnames) {
@@ -1948,6 +1746,16 @@ int verify_hostname(X509 *cert, const StringRef &hostname,
 
   return rv ? 0 : -1;
 }
+
+namespace {
+int verify_hostname(X509 *cert, const StringRef &hostname,
+                    const Address *addr) {
+  if (util::numeric_host(hostname.c_str())) {
+    return verify_numeric_hostname(cert, hostname, addr);
+  }
+
+  return verify_dns_hostname(cert, hostname);
+}
 } // namespace
 
 int check_cert(SSL *ssl, const Address *addr, const StringRef &host) {
@@ -2110,16 +1918,7 @@ int cert_lookup_tree_add_ssl_ctx(
     SSL_CTX *ssl_ctx) {
   std::array<uint8_t, NI_MAXHOST> buf;
 
-#if LIBRESSL_2_7_API ||                                                        \
-    (!LIBRESSL_IN_USE && OPENSSL_VERSION_NUMBER >= 0x10002000L)
   auto cert = SSL_CTX_get0_certificate(ssl_ctx);
-#else  // !LIBRESSL_2_7_API && OPENSSL_VERSION_NUMBER < 0x10002000L
-  auto tls_ctx_data =
-      static_cast<TLSContextData *>(SSL_CTX_get_app_data(ssl_ctx));
-  auto cert = load_certificate(tls_ctx_data->cert_file);
-  auto cert_deleter = defer(X509_free, cert);
-#endif // !LIBRESSL_2_7_API && OPENSSL_VERSION_NUMBER < 0x10002000L
-
   auto altnames = static_cast<GENERAL_NAMES *>(
       X509_get_ext_d2i(cert, NID_subject_alt_name, nullptr, nullptr));
   if (altnames) {
@@ -2383,23 +2182,19 @@ SSL_CTX *setup_downstream_client_ssl_context(
 #ifdef HAVE_NEVERBLEED
       nb,
 #endif // HAVE_NEVERBLEED
-      tlsconf.cacert, tlsconf.client.cert_file, tlsconf.client.private_key_file,
-      select_next_proto_cb);
+      tlsconf.cacert, tlsconf.client.cert_file,
+      tlsconf.client.private_key_file);
 }
 
 void setup_downstream_http2_alpn(SSL *ssl) {
-#if OPENSSL_VERSION_NUMBER >= 0x10002000L
   // ALPN advertisement
   auto alpn = util::get_default_alpn();
   SSL_set_alpn_protos(ssl, alpn.data(), alpn.size());
-#endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
 }
 
 void setup_downstream_http1_alpn(SSL *ssl) {
-#if OPENSSL_VERSION_NUMBER >= 0x10002000L
   // ALPN advertisement
   SSL_set_alpn_protos(ssl, NGHTTP2_H1_1_ALPN.byte(), NGHTTP2_H1_1_ALPN.size());
-#endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
 }
 
 std::unique_ptr<CertLookupTree> create_cert_lookup_tree() {
@@ -2422,8 +2217,8 @@ std::vector<uint8_t> serialize_ssl_session(SSL_SESSION *session) {
 } // namespace
 
 void try_cache_tls_session(TLSSessionCache *cache, SSL_SESSION *session,
-                           ev_tstamp t) {
-  if (cache->last_updated + 1_min > t) {
+                           const std::chrono::steady_clock::time_point &t) {
+  if (cache->last_updated + 1min > t) {
     if (LOG_ENABLED(INFO)) {
       LOG(INFO) << "Client session cache entry is still fresh.";
     }
@@ -2432,7 +2227,7 @@ void try_cache_tls_session(TLSSessionCache *cache, SSL_SESSION *session,
 
   if (LOG_ENABLED(INFO)) {
     LOG(INFO) << "Update client cache entry "
-              << "timestamp = " << t;
+              << "timestamp = " << t.time_since_epoch().count();
   }
 
   cache->session_data = serialize_ssl_session(session);
@@ -2468,9 +2263,7 @@ int proto_version_from_string(const StringRef &v) {
 
 int verify_ocsp_response(SSL_CTX *ssl_ctx, const uint8_t *ocsp_resp,
                          size_t ocsp_resplen) {
-
-#if !defined(OPENSSL_NO_OCSP) && !LIBRESSL_IN_USE &&                           \
-    OPENSSL_VERSION_NUMBER >= 0x10002000L
+#ifndef OPENSSL_NO_OCSP
   int rv;
 
   STACK_OF(X509) * chain_certs;
@@ -2516,11 +2309,7 @@ int verify_ocsp_response(SSL_CTX *ssl_ctx, const uint8_t *ocsp_resp,
     return -1;
   }
 
-#  if OPENSSL_1_1_API
   auto certid = OCSP_SINGLERESP_get0_id(sresp);
-#  else  // !OPENSSL_1_1_API
-  auto certid = sresp->certId;
-#  endif // !OPENSSL_1_1_API
   assert(certid != nullptr);
 
   ASN1_INTEGER *serial;
@@ -2547,8 +2336,7 @@ int verify_ocsp_response(SSL_CTX *ssl_ctx, const uint8_t *ocsp_resp,
   if (LOG_ENABLED(INFO)) {
     LOG(INFO) << "OCSP verification succeeded";
   }
-#endif // !defined(OPENSSL_NO_OCSP) && !LIBRESSL_IN_USE
-       // && OPENSSL_VERSION_NUMBER >= 0x10002000L
+#endif // !OPENSSL_NO_OCSP
 
   return 0;
 }
@@ -2593,13 +2381,6 @@ StringRef get_x509_issuer_name(BlockAllocator &balloc, X509 *x) {
   return get_x509_name(balloc, X509_get_issuer_name(x));
 }
 
-#ifdef WORDS_BIGENDIAN
-#  define bswap64(N) (N)
-#else /* !WORDS_BIGENDIAN */
-#  define bswap64(N)                                                           \
-    ((uint64_t)(ntohl((uint32_t)(N))) << 32 | ntohl((uint32_t)((N) >> 32)))
-#endif /* !WORDS_BIGENDIAN */
-
 StringRef get_x509_serial(BlockAllocator &balloc, X509 *x) {
   auto sn = X509_get_serialNumber(x);
   auto bn = BN_new();
@@ -2621,7 +2402,7 @@ namespace {
 int time_t_from_asn1_time(time_t &t, const ASN1_TIME *at) {
   int rv;
 
-#if OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL)
+#if defined(NGHTTP2_GENUINE_OPENSSL) || defined(NGHTTP2_OPENSSL_IS_LIBRESSL)
   struct tm tm;
   rv = ASN1_TIME_to_tm(at, &tm);
   if (rv != 1) {
@@ -2629,7 +2410,7 @@ int time_t_from_asn1_time(time_t &t, const ASN1_TIME *at) {
   }
 
   t = nghttp2_timegm(&tm);
-#else // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#else // !NGHTTP2_GENUINE_OPENSSL && !NGHTTP2_OPENSSL_IS_LIBRESSL
   auto b = BIO_new(BIO_s_mem());
   if (!b) {
     return -1;
@@ -2642,7 +2423,7 @@ int time_t_from_asn1_time(time_t &t, const ASN1_TIME *at) {
     return -1;
   }
 
-#  ifdef OPENSSL_IS_BORINGSSL
+#  ifdef NGHTTP2_OPENSSL_IS_BORINGSSL
   char *s;
 #  else
   unsigned char *s;
@@ -2655,18 +2436,14 @@ int time_t_from_asn1_time(time_t &t, const ASN1_TIME *at) {
   }
 
   t = tt;
-#endif // !(OPENSSL_1_1_1_API && !defined(OPENSSL_IS_BORINGSSL))
+#endif // !NGHTTP2_GENUINE_OPENSSL && !NGHTTP2_OPENSSL_IS_LIBRESSL
 
   return 0;
 }
 } // namespace
 
 int get_x509_not_before(time_t &t, X509 *x) {
-#if OPENSSL_1_1_API
   auto at = X509_get0_notBefore(x);
-#else  // !OPENSSL_1_1_API
-  auto at = X509_get_notBefore(x);
-#endif // !OPENSSL_1_1_API
   if (!at) {
     return -1;
   }
@@ -2675,11 +2452,7 @@ int get_x509_not_before(time_t &t, X509 *x) {
 }
 
 int get_x509_not_after(time_t &t, X509 *x) {
-#if OPENSSL_1_1_API
   auto at = X509_get0_notAfter(x);
-#else  // !OPENSSL_1_1_API
-  auto at = X509_get_notAfter(x);
-#endif // !OPENSSL_1_1_API
   if (!at) {
     return -1;
   }
