@@ -26,7 +26,7 @@
 #define UTIL_TEST_H
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#  include <config.h>
 #endif // HAVE_CONFIG_H
 
 namespace shrpx {
@@ -37,7 +37,6 @@ void test_util_inp_strlower(void);
 void test_util_to_base64(void);
 void test_util_to_token68(void);
 void test_util_percent_encode_token(void);
-void test_util_percent_encode_path(void);
 void test_util_percent_decode(void);
 void test_util_quote_string(void);
 void test_util_utox(void);
@@ -67,6 +66,9 @@ void test_util_format_hex(void);
 void test_util_is_hex_string(void);
 void test_util_decode_hex(void);
 void test_util_extract_host(void);
+void test_util_split_hostport(void);
+void test_util_split_str(void);
+void test_util_rstrip(void);
 
 } // namespace shrpx
 
